/* feature.h: features used in the lunar colongitude program.
 *
 * Copyright 2003 by Akkana Peck, akkana <at> shallowsky.com.
 * You are free to free to use or modify this software under the GPL.
 */

#include "feature.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

/* This is declared in string.h but for some reason gcc isn't seeing it */
extern char *strndup (char *__string, size_t __n);

struct LunarFeature* ReadFeature(char* line)
{
    double colong;
    int page;
    struct LunarFeature* feature;
    char* cp;

    while (isspace(*line) && *line) ++line;   /* skip leading whitespace */

    /* Read the colongitude */
    if (!*line || *line == '\n')
        return 0;
    if (!isdigit(*line) && *line != '-' && *line != '+')
        return 0;
    colong = atof(line);

    while (!isspace(*line) && *line) ++line;  /* skip over the number */
    while (isspace(*line) && *line) ++line;   /* skip intervening whitespace */
    if (*line == 0) return 0;

    /* Read the Rukl page */
    if (!*line || *line == '\n') return 0;
    if (!isdigit(*line) && *line != '-' && *line != '+')
        return 0;
    page = atoi(line);

    while (!isspace(*line) && *line) ++line;  /* skip over the number */
    while (isspace(*line) && *line) ++line;   /* skip intervening whitespace */
    if (!*line || *line == '\n') return 0;

    /* Now we have everything, so it's safe to allocate the new feature */
    feature = (struct LunarFeature*)malloc(sizeof (struct LunarFeature));
    if (!feature) return 0;

    /* strip the newline, if any */
    for (cp = line; *cp; ++cp)
        if (*cp == '\n') {
            *cp = '\0';
            break;
        }

    feature->longdesc = strdup(line);
    if (!feature->longdesc) {
        free(feature);
        return 0;
    }
    feature->colong = colong;
    feature->page = page;
    return feature;
}

void FreeFeature(struct LunarFeature* feature)
{
    if (!feature) return;
    if (feature->longdesc)
        free(feature->longdesc);
    free(feature);
}

static int strcasestr(const char* longstr, const char* substr)
{
    register const char* s = substr;
    while (*longstr && *s)
    {
        if (tolower(*longstr) == tolower(*s))
        {
            ++longstr;
            ++s;
            continue;
        }
        if (s == substr)
            ++longstr;
        s = substr;
    }
    return (*s == 0);
}

struct LunarFeature*
GetFeatureByKeyword(const char* keyword, const char* filename)
{
    FILE* fp = fopen(filename, "r");
    char line[BUFSIZ];
    if (!fp) {
        perror(filename);
        return 0;
    }
    while (fgets(line, sizeof line, fp) != 0) {
        struct LunarFeature* feature = ReadFeature(line);
        if (!feature || !feature->longdesc) continue;

        if (strcasestr(feature->longdesc, keyword))
            return feature;

        FreeFeature(feature);
    }
    return 0;
}


